<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * The Controller to call after clicking Place Order using Montonio
 */
class MontonioValidationModuleFrontController extends ModuleFrontController
{

    /**
     * @var boolean
     */
    public $ssl = true;

    /**
     * Redirect to Montonio Payments and commence payment flow variant (2) as described at
     * https://devdocs.prestashop.com/1.7/modules/sell/techvalidation-checklist/#payment-modules-rules
     *
     * @return void
     */
    public function postProcess()
    {
        /**
         * Redirect back to cart if initial validation failed
         */
        if (!$this->performValidations()) {
            Tools::redirect($this->context->link->getPageLink('order'));
            die();
        }

        /**
         * Create payment URL and redirect the customer to Montonio
         */
        $method      = Tools::getValue('method');

        try{
        $paymentData = $this->module->callPaymentFunc($method, 'preparePaymentData', $this->context, $this->module);
        } catch (\Exception $e) {
            //@TODO clone the cart and restore it or abandon with some message
            Tools::redirect($this->context->link->getPageLink('order'));
            die();
        }

        $paymentUrl  = $this->module->callPaymentFunc($method, 'getRedirectUrl', $paymentData);

        Tools::redirect($paymentUrl);
        die();
    }

    /**
     * Perform the initial Prestashop-related validations
     *
     * @return bool True if validations passed, false otherwise
     */
    protected function performValidations()
    {
        $cart = $this->context->cart;

        /**
         * Module inactive
         */
        if (!$this->module->active) {
            return false;
        }

        /**
         * Unsupported Montonio payment method or not enabled
         */
        $method = Tools::getValue('method');
        $methods = ['MontonioFinancing', 'MontonioPayments', 'MontonioBlikPayments', 'MontonioSplit'];
        if (!in_array($method, $methods) ||
            !$this->module->callPaymentFunc($method, 'isEnabled')) {
            return false;
        }

        /**
         *  Unsupported currency
         */
        if (!$this->module->isCurrencySupported($method)) {
            return false;
        }

        /**
         * Problem with identifying customer information
         */
        if ($cart->id_customer == 0 ||
            $cart->id_address_delivery == 0 ||
            $cart->id_address_invoice == 0
        ) {
            return false;
        }

        /**
         * Check if Montonio is a payment module
         */
        $authorized     = false;
        $paymentModules = Module::getPaymentModules();
        foreach ($paymentModules as $module) {
            if ($module['name'] == $this->module->name) {
                $authorized = true;
                break;
            }
        }
        if (!$authorized) {
            $this->errors[] = $this->module->l('This payment method is not available.', 'validation');
            return false;
        }

        /**
         * Validate customer is loaded
         */
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            return false;
        }

        return true; // passed validations
    }
}
